
#include "ai.h"
#include "player.h"
#include "order.h"
#include "point.h"
#include "time.h"
#include "fifoStack.h"
#include "config.h"

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <allegro.h>

#define VISION (PLAYER_RADIUS * 10)

//! Artificial Intelligence Functions

AI::AI ()
{
	myState = WALKING_AROUND;
	hasDestination = false;
}

void AI::Init (PlayerArray* players, short id)
{
	pa = players;
	bot = pa->getPlayer(id);
	catcher = pa->getCatcher();
}

void AI::updateCatcher (Player* p)
{
	catcher = p;
}

bool AI::iCanSee (Player* other)
{
	return ( bot->center.sqrDistance(&other->center) < (VISION * VISION) );
}

void AI::updateVision ()
{
	visionCount = 0;

	if (bot->isCatcher())
	{
		// Catcher
		for (int i = 0; i < pa->size; i++)
		{
			Player* p = pa->getPlayer(i);
			if ((p != bot) && iCanSee(p))
			{
				visionArea[visionCount++] = p;
			}
		}
	}
	else
	{
		if (!catcher) return;
		// Runner
		if (iCanSee (catcher))
		{
			visionArea[visionCount++] = catcher;
		}
	}
}

void AI::WalkAroundState (FifoStack &stack, short turn)
{
	if (!hasDestination)
	{
		srand(time(NULL));
		dest.setX(rand()%XBUFFER);
		dest.setY(rand()%YBUFFER);
	}

	Point idealSpeed = dest - bot->center;

	if (bot->speed.getX() < idealSpeed.getX())
	{
		stack.Push(new Order(Order::MOVE_RIGHT, bot->id, turn));
	}
	else
	{
		stack.Push(new Order(Order::MOVE_LEFT, bot->id, turn));
		
	}

	if (bot->speed.getY() < idealSpeed.getY())
	{
		stack.Push(new Order(Order::MOVE_BACKWARD, bot->id, turn));
	}
	else
	{
		stack.Push(new Order(Order::MOVE_FOWARD, bot->id, turn));
	}
}

void AI::HuntState (FifoStack &stack, short turn)
{
	// TODO: coming soon...
	Player* p = visionArea[0];

	Point idealSpeed = p->center - bot->center;

	if (bot->speed.getX() < idealSpeed.getX())
	{
		stack.Push(new Order(Order::MOVE_RIGHT, bot->id, turn));
	}
	else
	{
		stack.Push(new Order(Order::MOVE_LEFT, bot->id, turn));
		
	}

	if (bot->speed.getY() < idealSpeed.getY())
	{
		stack.Push(new Order(Order::MOVE_BACKWARD, bot->id, turn));
	}
	else
	{
		stack.Push(new Order(Order::MOVE_FOWARD, bot->id, turn));
	}
}

void AI::RunState (FifoStack &stack, short turn)
{
	// TODO: coming soon...
	stack.Push(new Order(Order::NULL_ORDER, bot->id, turn));
}

void AI::RunAI (FifoStack &stack, short turn)
{
	updateVision();

	if (!visionCount)
	{
		// If i can't see my opponent
		myState = WALKING_AROUND;
	}
	else if (bot->isCatcher())
	{
		// If i can see and i'm a catcher
		myState = HUNTING;
	}
	else
	{
		// If i can see and i'm a runner
		myState = RUNNING;
	}

	switch (myState)
	{
		case WALKING_AROUND:
			WalkAroundState(stack, turn);
			break;

		case HUNTING:
			HuntState(stack, turn);
			break;

		case RUNNING:
			RunState(stack, turn);
			break;
	}
}
