
#include "customSocket.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <netdb.h>

#include <fcntl.h>

//! BaseServer Functions

BaseServer::BaseServer()
{
	sin_size = sizeof(struct sockaddr_in);
	size = 1;
}

bool BaseServer::makeServer (int port)
{
	int yes = 1;

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
		exit(1);
	}

	if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
	{
		perror("setsockopt");
		exit(1);
	}
	
	my_addr.sin_family = AF_INET;		 // host byte order
	my_addr.sin_port = htons(port);	 // short, network byte order
	my_addr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
	memset(&(my_addr.sin_zero), '\0', 8); // zero the rest of the struct

	if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) == -1)
	{
		perror("bind");
		exit(1);
	}

	if (listen(sockfd, 2) == -1)
	{
		perror("listen");
		exit(1);
	}

	// So the accept() command can return, even if theres no pending connections
	fcntl(sockfd, F_SETFL, O_NONBLOCK);

	return true;
}

bool BaseServer::Accept()
{
	if ((new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size)) == -1)
	{
		//perror("accept");
		return false;
	}

	printf("Server: got connection from %s\n",inet_ntoa(their_addr.sin_addr));
	short index = nextConnection();
	connectionArray[index].socket = new_fd;
	connectionArray[index].connectionId = index;
	connectionArray[index].active = true;

	lastConnectionId = index;

	// So the recv() command can return, even if theres no pending messages
	fcntl(connectionArray[index].socket, F_SETFL, O_NONBLOCK);

	return true;
}

void BaseServer::Send(void* buffer, int bufferSize, int index)
{
	if (!connectionArray[index].active) return;
	send(connectionArray[index].socket, buffer, bufferSize, 0);
}

void BaseServer::SendToAll(void* buffer, int bufferSize)
{
	for (int i = 1; i < size; i++)
	{
		if (connectionArray[i].active)
			send(connectionArray[i].socket, buffer, bufferSize, 0);
	}
}

void BaseServer::SendToAllExcept(void* buffer, int bufferSize, int id)
{
	for (int i = 1; i < size; i++)
	{
		if (connectionArray[i].active && (i != id))
			send(connectionArray[i].socket, buffer, bufferSize, 0);
	}
}

int BaseServer::Receive(void* buffer, int bufferSize, int index)
{
	if (!connectionArray[index].active) return -1;
	return recv(connectionArray[index].socket, buffer, bufferSize, 0);
}

short BaseServer::nextConnection()
{
	int i;
	for (i = 1; i < size; i++)
	{
		if (!connectionArray[i].active)
		{
			return i;
		}
	}
	size++;
	return i;
}

void BaseServer::deactivateConnection(int connectionId)
{
	int i;
	for (i = 1; i < size; i++)
	{
		if (connectionArray[i].connectionId == connectionId)
		{
			connectionArray[i].active = false;
			break;
		}
	}

	if (connectionId == (size -1)) size--;
}

//! BaseClient Functions

BaseClient::BaseClient()
{
	mySocket = 0;
}

bool BaseClient::makeClient(int port, char* host)
{
	struct hostent *he;
	struct sockaddr_in their_addr; // connector's address information 

	if ((he=gethostbyname(host)) == NULL) {  // get the host info 
	    perror("gethostbyname");
	    return false;
	    // exit(1);
	}

	if ((mySocket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	    perror("socket");
	    return false;
	    // exit(1);
	}

	their_addr.sin_family = AF_INET;    // host byte order 
	their_addr.sin_port = htons(port);  // short, network byte order 
	their_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct 

	if (connect(mySocket, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) {
	    perror("connect");
	    return false;
	    // exit(1);
	}

	// So the recv() command can return, even if theres no pending messages
	fcntl(mySocket, F_SETFL, O_NONBLOCK);
	
	return true;
}

int BaseClient::Send(void* buffer, int size)
{
	return send(mySocket, buffer, size, 0);
}

int BaseClient::Receive(void* buffer, int size)
{
	return recv(mySocket, buffer, size, 0);
}
