
#include "engine.h"
#include "gui.h"
#include "player.h"
#include "netGame.h"
#include "config.h"

#include <allegro.h>
#include <stdlib.h>

// Engine Constructor
Engine::Engine()
{
	inGame = true;
}

// Executes an Order
void Engine::executeOrder(Order* myOrder)
{
	if ( (myOrder->type == Order::QUIT || myOrder->type == Order::PLAYER_KICKED) )
	{
		pa.executeOrder(myOrder);

		if (myOrder->playerId == gui.playerId)
		{
			// It's myself
			inGame = false;
		}
		else
		{
			// Another player
			pa.deactivatePlayer(myOrder->playerId);
			// Server will need this
			quit = true;
			quitterId = myOrder->playerId;
			if (myOrder->type == Order::QUIT)
			{
				printf("A player left (totalPlayers: %d - paSize: %d)\n", pa.totalPlayers, pa.size);
			}
			else
			{
				printf("A player got kicked for lagging.\n");
			}
		}
	}
	else
	{
		pa.executeOrder(myOrder);
		quit = false;
	}
}

// Collision and movement
void Engine::runPhysics()
{
	for (int i = 0; i < pa.size; i++)
	{
		if (pa.getPlayer(i)->inGame)
		{
			pa.getPlayer(i)->Move();
		}
	}
}

void Engine::runLogic()
{
	Player* myPlayer;
	Player* catcher;

	// Decrease the immune timer
	for (int i = 0; i < pa.size; i++)
	{
		myPlayer = pa.getPlayer(i);

		if (myPlayer->inGame && myPlayer->isImmune())
		{
			myPlayer->decImmuneTimer();
		}
	}

	// Finds the catcher
	catcher = pa.getCatcher();

	// Set a catcher if there's none
	if (!catcher)
	{
		// NOTE: Provisory: Server will be the catcher
		catcher = pa.getPlayer(0);
		// He was not catched, so i won't increase the variable
		catcher->setCatcher();
		if (type == ARTIFICIAL)
		{
			gui.updateCatcher(catcher);
		}
	}

	// Check the catcher's collision
	for (int i = 0; i < pa.size; i++)
	{
		myPlayer = pa.getPlayer(i);

		if (myPlayer->inGame && myPlayer->isRunner() && (myPlayer->id != catcher->id))
		{
			if (catcher->checkCollision(myPlayer))
			{
				// Change status
				catcher->setImmune();
				myPlayer->setCatcher();
				myPlayer->beingCatched++;
				break;
			}
		}
	}
}
