
#include "engine.h"
#include "gui.h"
#include "player.h"
#include "netGame.h"
#include "config.h"
#include "engineServer.h"

#include <allegro.h>
#include <stdlib.h>

// EngineServer Constructor
EngineServer::EngineServer(int port, char* name, bool human)
{
	gui.turn = DELAY;
	gui.playerId = pa.createNewPlayer();
	ngs.initializeMyStack(gui.playerId);
	// Server always has id 0
	pa.getPlayer(0)->inGame = true;
	pa.getPlayer(0)->setName(name);

	ngs.myServer.makeServer(port);
	
	type = human ? HUMAN : ARTIFICIAL;
	if (type == ARTIFICIAL)
	{
		gui.setAI(&pa);
	}
}

// Well...
void EngineServer::receiveNewPlayers()
{
	if (ngs.Accept(&pa, gui.turn))
	{
		printf("Step: %d Gui-Turn: %d\n", ngs.step, gui.turn);
	}
}

//! Main Server Function
void EngineServer::oneTurn()
{
	if (!inGame) return;

	if (!ngs.step)
	{
		ngs.checkStack();
	}

	receiveNewPlayers();

	gui.handleEvents();

	ngs.SendOrders(gui.orderStack);
	ngs.ReceiveAndSendOrders();

	// For every player in the game
	for (int p = 0; p < pa.size; p++)
	{
		// Execute orders in the step
		if (pa.getPlayer(p)->inGame && (!ngs.hasOrders(p)))
		{
			printf("Player %d did not send orders\n", pa.getPlayer(p)->id);
			printf("step: %d\n", ngs.step);
			// TODO: I would probably use another stack for server orders like kick player, map change etc
			// Player will be kicked on the next gui turn
			gui.orderStack.Push(new Order(Order::PLAYER_KICKED, pa.getPlayer(p)->id, gui.turn+1));
			// So this if statement won't be done on the next turn
			pa.getPlayer(p)->inGame = false;
		}

		while (ngs.hasOrders(p))
		{
			Order* currentOrder = ngs.Pop(p);
			// Player does not need to be in game (Since there's a PLAYER_JOINED order)
			executeOrder(currentOrder);
			delete currentOrder;
			if (quit)
			{
				printf("Someone just quitted\n");
				ngs.myServer.deactivateConnection(quitterId);
				quit = false;
			}
		}
	}
	// printf("Executed\n");

	runPhysics();
	runLogic();
	gui.Draw(&pa);

	ngs.nextStep();
	gui.nextTurn();
}

// Auxiliary for allegro timer
void RunServer(void* myEngineServer)
{
	EngineServer* that = (EngineServer*) myEngineServer;
	that->oneTurn();
}
END_OF_FUNCTION(RunServer);

void EngineServer::Run()
{
	LOCK_FUNCTION(RunServer);
	install_param_int(RunServer, (void*)this, TURN_TIME);
	while(inGame)
	{
		rest(TURN_TIME);
	}
	remove_param_int(RunServer, (void*)this);
}
