
#include "player.h"
#include "point.h"
#include "time.h"
#include "config.h"

#include <math.h>
#include <stdio.h>
#include <string.h>

//! Player Functions

Player::Player()
{

}

void Player::Create(int playerId)
{
	srand(time(NULL));
	id = playerId;
	inGame = false;
	setRunner();
	center.setVector((rand()%120*5), (rand()%90*5));
	speed.setVector(0, 0);
	beingCatched = 0;
	name[0] = 0;
}

void Player::setName(char* playerName)
{
	strcpy(name, playerName);
}

// Get the X blit position
int Player::getX()
{
	return (int) center.getX();
}

// Get the Y blit position
int Player::getY()
{
	return (int) center.getY();
}

void Player::addXSpeed(float value)
{
	speed.setX(speed.getX()+value);
}

void Player::addYSpeed(float value)
{
	speed.setY(speed.getY()+value);
}

void Player::executeOrder(Order* myOrder)
{
	if (myOrder->type == Order::MOVE_FOWARD)
	{
		float value = isCatcher() ? -ACCELERATION*2 : -ACCELERATION;
		addYSpeed(value);
	}
	else if (myOrder->type == Order::MOVE_BACKWARD)
	{
		float value = isCatcher() ? ACCELERATION*2 : ACCELERATION;
		addYSpeed(value);
	}
	else if (myOrder->type == Order::MOVE_LEFT)
	{
		float value = isCatcher() ? -ACCELERATION*2 : -ACCELERATION;
		addXSpeed(value);
	}
	else if (myOrder->type == Order::MOVE_RIGHT)
	{
		float value = isCatcher() ? ACCELERATION*2 : ACCELERATION;
		addXSpeed(value);
	}
	else if (myOrder->type == Order::PLAYER_JOINED)
	{
		inGame = true;
	}
	else if (myOrder->type == Order::QUIT)
	{
		inGame = false;
	}
	else if (myOrder->type == Order::NULL_ORDER)
	{
		// Well...
	}
}

void Player::Attrition()
{
	float attrict = ATTRICT;
	Vector auxSpeed = speed;

	// Get the angle between speed and origin
	Vector v; // Null Vector
	float angle = v.calculateAngle(&auxSpeed);

	// Rotate the speed, so it is over the X axis
	auxSpeed.vectorRotation(angle);
	float oldX = auxSpeed.getX();

	// Check if it is beyond the speed limits
	float maxSpeed = isCatcher() ? MAX_CATCHER_SPEED : MAX_SPEED;
	if (oldX < -maxSpeed)
	{
		oldX = -maxSpeed;
	}
	else if (oldX > maxSpeed)
	{
		oldX = maxSpeed;
	}

// 	if (oldX != 0)
// 	{
// 		printf("Angle: %f\n", angle);
// 		printf("oldX: %f\n", oldX);
// 	}

	if (oldX == 0)
	{
		// Speed is already zero
		return;
	}
	else if (oldX > 0)
	{
		auxSpeed.setX(oldX - attrict);
	}
	else
	{
		auxSpeed.setX(oldX + attrict);
	}

	if (oldX * auxSpeed.getX() < 0)
	{
		// If the attrict reverted the speed direction
		// Reset speed
		speed.setVector(0, 0);
	}
	else
	{
		// Update speed
		auxSpeed.vectorRotation(-angle);
		speed = auxSpeed;
	}
}

void Player::WindowCollision()
{
	// NOTE: This is provisory stuff
	if (center.getX() < PLAYER_RADIUS + 5) { center.setX(PLAYER_RADIUS + 5); speed.setX(-1*speed.getX()); }
	if (center.getX() > XBUFFER - PLAYER_RADIUS - 5) { center.setX(XBUFFER - PLAYER_RADIUS - 5); speed.setX(-1*speed.getX()); }
	if (center.getY() < PLAYER_RADIUS + 5) { center.setY(PLAYER_RADIUS + 5); speed.setY(-1*speed.getY()); }
	if (center.getY() > YBUFFER - PLAYER_RADIUS - 5) { center.setY(YBUFFER - PLAYER_RADIUS - 5); speed.setY(-1*speed.getY()); }
}

void Player::Move()
{
	// Basic attrict and speed fix
	Attrition();
	center.addVector(&speed);
	WindowCollision();
}

bool Player::isCatcher()
{
	return (status == CATCHER);
}

bool Player::isRunner()
{
	return (status == RUNNER);
}

bool Player::isImmune()
{
	return (status == IMMUNE);
}

void Player::setCatcher()
{
	status = CATCHER;
}

void Player::setImmune()
{
	immuneTimer = 60;
	status = IMMUNE;
}

void Player::setRunner()
{
	status = RUNNER;
}

bool Player::checkCollision(Player* other)
{
	return (center.sqrDistance(&other->center) <= (4*PLAYER_RADIUS*PLAYER_RADIUS));
}

void Player::decImmuneTimer()
{
	immuneTimer--;
	if (immuneTimer <= 0) setRunner();
}

//! PlayerArray Functions

PlayerArray::PlayerArray ()
{
	size = 0;
	totalPlayers = 0;
	for (int i = 0; i < MAX_PLAYERS; i++)
	{
		available[i] = true;
	}
}

void PlayerArray::deactivatePlayer (int index)
{
	if (index == size) size--;
	totalPlayers--;
	available[index] = true;
}

short PlayerArray::emptyPosition ()
{
	int i;
	for (i = 0; i < size; i++)
	{
		if (available[i])
		{
			return i;
		}
	}
	return i;
}

short PlayerArray::createNewPlayer ()
{
	short index = emptyPosition();
	array[index].Create(index);

	if (index == size) size++;
	totalPlayers++;

	available[index] = false;

	return index;
}

void PlayerArray::executeOrder (Order* myOrder)
{
	array[myOrder->playerId].executeOrder(myOrder);
}

Player* PlayerArray::getPlayer (int index)
{
	return &array[index];
}

void PlayerArray::setPlayer (Player myPlayer)
{
	array[myPlayer.id] = myPlayer;
	if (myPlayer.id >= size) size = myPlayer.id + 1;
}

void PlayerArray::setNumPlayers (short num)
{
	totalPlayers = num;
	size = num;
}

Player* PlayerArray::getCatcher ()
{
	for (int i = 0; i < size; i++)
	{
		Player* p = getPlayer(i);
		if (p->isCatcher() && p->inGame)
		{
			return p;
		}
	}
	return NULL;
}
