
#include "point.h"

#include <math.h>

//! Vector Methods

// Constructors
Vector::Vector ()
{
	x = 0.0f;
	y = 0.0f;
}

Vector::Vector (float x0, float y0)
{
	x = x0;
	y = y0;
}

// Read properties
float Vector::getX ()
{
	return x;
}

float Vector::getY ()
{
	return (y);
}

int Vector::getXInt ()
{
	return (int) x;
}

int Vector::getYInt ()
{
	return (int) y;
}

// Write properties
void Vector::setX (float x)
{
	this->x = x;
}

void Vector::setY (float y)
{
	this->y = y;
}

void Vector::setVector (float x, float y)
{
	this->x = x;
	this->y = y;
}

void Vector::setVector (Vector* v)
{
	x = v->getX();
	y = v->getY();
}

void Vector::addVector (Vector* v)
{
	x += v->x;
	y += v->y;
}

void Vector::subVector (Vector* v)
{
	x -= v->x;
	y -= v->y;
}

void Vector::mulVector (Vector* v)
{
	x *= v->x;
	y *= v->y;
}

// Anti-Clockwise Rotation
void Vector::vectorRotation (float angle)
{
	float oldX;
	oldX = x;
	x = (sin(angle) * y + cos(angle) * x);
	y = (cos(angle) * y - sin(angle) * oldX);
}

// Calculate the angle between our Point and another Point (Related to the X axis)
float Point::calculateAngle (Point* p)
{
	float angle;

	Point aux1(x, y);
	aux1.subVector(p);

	angle = atan2(aux1.getY(), aux1.getX());
	return angle;
}

// Square of the distance between our Point and another Point
float Point::sqrDistance (Point* p)
{
	return ( (x - p->x) * (x - p->x) + (y - p->y) * (y - p->y) );
}

Vector Vector::operator - (Vector &v)
{
	Vector temp;
	temp.x = x - v.x;
	temp.y = y - v.y;
	return temp;
}
